#include <iostream>
#include <event2/event.h>
#include <event2/thread.h>
#include <event2/listener.h>
#include <event2/bufferevent.h>
#include <event2/buffer.h>

#ifdef _WIN32
#include <string>
#else
#include <string.h>
#include <signal.h>
#endif // !_WIN32

using namespace std;

#define SPORT 5001
struct Status {
	bool start = false;
	FILE *fp = 0;
	string filename = "";
};
bufferevent_filter_result server_filter_in(struct evbuffer *src, struct evbuffer *dst, ev_ssize_t dst_limit,
	enum bufferevent_flush_mode mode, void *ctx)
{
	cout << "server_filter_in" << endl;

	//1.接收客户端发送的文件名
	char data[1024] = { 0 };
	int len = evbuffer_remove(src, data, sizeof(data) - 1);
	evbuffer_add(dst, data, len);
	return BEV_OK;
}
void server_read_cb(bufferevent *bev, void *arg) {

	Status *status = (Status*)arg;
	if (!status->start) {
		char data[1024] = { 0 };
		bufferevent_read(bev, data, sizeof(data) - 1);
		status->filename = data;
		string out = "out\\";
		out += data;
		status->fp = fopen(out.c_str(), "wb");
		if (!status->fp) {
			cout << "open file " << out << "faild！" << endl;
			return;
		}

		bufferevent_write(bev, "ok", 2);
		status->start = true;
		return;
	}

	//写入文件
	do {
		char data[1024] = { 0 };
		int len = bufferevent_read(bev, data, sizeof(data) - 1);
		if (len >= 0) {
			fwrite(data, 1, len, status->fp);
			fflush(status->fp);
		}
	} while (evbuffer_get_length(bufferevent_get_input(bev)) > 0);
	
}
void server_event_cb(bufferevent *bev, short events, void *arg) {
	cout << "server_event_cb" << events << endl;

	Status *status = (Status *)arg;
	if (events&BEV_EVENT_EOF || events & BEV_EVENT_TIMEOUT) {
		if (status->fp) {
			fclose(status->fp);
			status->fp = 0;
		}
		bufferevent_free(bev);
	}
}


void server_listen_cb(struct evconnlistener *e, evutil_socket_t s, struct sockaddr *a, int socklen, void *arg)
{
	std::cout << "server_listen_cb" << std::endl;
	event_base *base = (event_base *)arg;
	// 1.创建bufferevent 用于通信
	bufferevent *bev = bufferevent_socket_new(base, s, BEV_OPT_CLOSE_ON_FREE);
	// 2.添加过滤 并设置输入回调
	Status *status = new Status;
	bufferevent *bev_filter = bufferevent_filter_new(bev,
		server_filter_in,//输入过滤函数
		0, //输出过滤
		BEV_OPT_CLOSE_ON_FREE, //关闭filter同时关闭bufferevent
		0, //清理回调
		status); //传递参数
	// 3.设置回调  读取 事件（处理连接断开） 
	bufferevent_setcb(bev_filter, server_read_cb, 0, server_event_cb, status);
	bufferevent_enable(bev_filter, EV_READ | EV_WRITE);
}


void Server(event_base* base) {
	cout << "begin Server" << endl;

	sockaddr_in sin;
	memset(&sin, 0, sizeof(sin));
	sin.sin_family = AF_INET;
	sin.sin_port = htons(SPORT);

	evconnlistener *ev = evconnlistener_new_bind(base,										//libevent上下文
		server_listen_cb,									//接收到连接的回调函数
		base,										//回调函数获取的参数(根据业务来)
		LEV_OPT_REUSEABLE | LEV_OPT_CLOSE_ON_FREE, //地址重用，evconnlistener关闭同时关闭socket
		10,										//连接队列大小，对应listen函数参数
		(sockaddr *)&sin,							//绑定的地址和端口
		sizeof(sin));
}